@@ -4,13 +4,16 @@ use strict;
 use base 'HTML::FormFu::Element::Button';
 use Class::C3;
 
-__PACKAGE__->mk_attr_accessors(qw/ src width height /);
+__PACKAGE__->mk_attr_accessors(qw( src width height ));
 
 sub new {
     my $self = shift->next::method(@_);
 
     $self->field_type('image');
-    $self->src('') if !defined $self->src;
+
+    if ( !defined $self->src ) {
+        $self->src('');
+    }
 
     return $self;
 }
@@ -41,7 +44,7 @@ L<HTML::FormFu::Element::_Input>,
 L<HTML::FormFu::Element::_Field>, 
 L<HTML::FormFu::Element>
 
-L<HTML::FormFu::FormFu>
+L<HTML::FormFu>
 
 =head1 AUTHOR
 
