@@ -2,11 +2,17 @@ package HTML::FormFu::Element::Repeatable;
 
 use strict;
 use base 'HTML::FormFu::Element::Block';
-use Class::C3;
-use Carp qw/ croak /;
 
-__PACKAGE__->mk_accessors(
-    qw/ _original_elements increment_field_names counter_name /);
+use HTML::FormFu::Util qw( DEBUG_PROCESS debug );
+use Class::C3;
+use List::Util qw( first );
+use Carp qw( croak );
+
+__PACKAGE__->mk_item_accessors( qw(
+        _original_elements
+        increment_field_names
+        counter_name
+) );
 
 sub new {
     my $self = shift->next::method(@_);
@@ -24,7 +30,7 @@ sub repeat {
     $count ||= 1;
 
     croak "invalid number to repeat"
-        unless $count =~ /^[1-9][0-9]*\z/;
+        if $count !~ /^[1-9][0-9]*\z/;
 
     my $children;
 
@@ -35,6 +41,7 @@ sub repeat {
     }
     else {
         $children = $self->_elements;
+
         $self->_original_elements($children);
     }
 
@@ -61,8 +68,9 @@ sub repeat {
                 if ( defined( my $name = $field->name ) ) {
                     $field->original_name($name);
 
-                    $name .= "_$rep";
-                    $field->name($name);
+                    $field->original_nested_name( $field->nested_name );
+
+                    $field->name("${name}_$rep");
                 }
             }
         }
@@ -76,22 +84,31 @@ sub repeat {
                 @{ $field->_constraints },
                 @{ $field->_inflators },
                 @{ $field->_validators },
-                @{ $field->_transformers };
+                @{ $field->_transformers },
+                @{ $field->_plugins },
+                ;
         }
 
         my $block_fields = $block->get_fields;
 
-        my @others_constraints = grep { $_->can('others') }
-            map { @{ $_->_constraints } } @$block_fields;
+        my @block_constraints = map { @{ $_->get_constraints } } @$block_fields;
+
+        # rename any 'others' fields
+        my @others_constraints = grep { defined $_->others }
+            grep { $_->can('others') } @block_constraints;
 
         for my $constraint (@others_constraints) {
             my $others = $constraint->others;
-            $others = [$others] if !ref $others;
+            if ( !ref $others ) {
+                $others = [$others];
+            }
             my @new_others;
 
             for my $name (@$others) {
-                my ($field)
-                    = grep { $_->original_name eq $name } @$block_fields;
+                my $field
+                    = ( first { $_->original_nested_name eq $name }
+                    @$block_fields )
+                    || first { $_->original_name eq $name } @$block_fields;
 
                 if ( defined $field ) {
                     push @new_others, $field->nested_name;
@@ -104,6 +121,21 @@ sub repeat {
             $constraint->others( \@new_others );
         }
 
+        # rename any 'when' fields
+        my @when_constraints = grep { defined $_->when } @block_constraints;
+
+        for my $constraint (@when_constraints) {
+            my $when = $constraint->when;
+            my $name = $when->{field};
+
+            my $field
+                = first { $_->original_nested_name eq $name } @$block_fields;
+
+            if ( defined $field ) {
+                $when->{field} = $field->nested_name;
+            }
+        }
+
         push @return, $block;
 
     }
@@ -132,11 +164,14 @@ sub process {
     if ( defined $self->counter_name && defined $form->query ) {
         my $input = $form->query->param( $self->counter_name );
 
-        $count = $input
-            if defined $input && $input =~ /^[1-9][0-9]*\z/;
+        if ( defined $input && $input =~ /^[1-9][0-9]*\z/ ) {
+            $count = $input;
+        }
     }
 
-    unless ( $self->_original_elements ) {
+    if ( !$self->_original_elements ) {
+        DEBUG_PROCESS && debug("calling \$repeatable->count($count)");
+
         $self->repeat($count);
     }
 
@@ -184,6 +219,7 @@ HTML::FormFu::Element::Repeatable - repeatable block element
     ---
     elements:
       - type: Repeatable
+        name: my_rep
         elements:
           - name: foo
           - name: bar
@@ -191,14 +227,34 @@ HTML::FormFu::Element::Repeatable - repeatable block element
 Calling C<< $element->repeat(2) >> would result in the following markup:
 
     <div>
-        <input name="foo" type="text" />
-        <input name="bar" type="text" />
+        <input name="my_rep.foo_1" type="text" />
+        <input name="my_rep.bar_1" type="text" />
     </div>
     <div>
-        <input name="foo" type="text" />
-        <input name="bar" type="text" />
+        <input name="myrep.foo_2" type="text" />
+        <input name="myrep.bar_2" type="text" />
     </div>
 
+Example of constraints:
+
+    ----
+    elements:
+      - type: Repeatable
+        name: my_rep
+        elements:
+          - name: id
+          
+          - name: foo
+            constraints:
+              - type: Required
+                when:
+                  field: 'my_rep.id' # use full nested-name
+          
+          - name: bar
+            constraints:
+              - type: Equal
+                others: 'my_rep.foo' # use full nested-name
+
 =head1 DESCRIPTION
 
 Provides a way to extend a form at run-time, by copying and repeating its 
@@ -213,6 +269,13 @@ elements in a new L<Block|HTML::FormFu::Element::Block> element, which
 inherits the Repeatable's display settings, such as L</attributes> and 
 L</tag>.
 
+For all constraints attached to fields within a Repeatable block which use
+either L<others|HTML::FormFu::Constraint::_others/others> or
+L<when|HTML::FormFu::Constraint/when> containing names of fields within
+the same Repeatable block, when L<repeat> is called, those names will
+automatically be updated to the new nested-name for each field (taking
+into account L<increment_field_names>).
+
 =head1 METHODS
 
 =head2 repeat
@@ -260,29 +323,8 @@ Arguments: $bool
 
 Default Value: 1
 
-If true, then any copies of fields whose name contains a C<0>, will have 
-the C<0> replaced by it's L</repeatable_count> value.
-
-    ---
-    elements:
-      - type: Repeatable
-        increment_field_names: 1
-        elements:
-          - name: foo_0
-          - name: bar_0
-
-Calling C<< $element->repeat(2) >> would result in the following markup:
-
-    <div>
-        <input name="foo_1" type="text" />
-        <input name="bar_1" type="text" />
-    </div>
-    <div>
-        <input name="foo_2" type="text" />
-        <input name="bar_2" type="text" />
-    </div>
-
-See also L</counter_name>.
+If true, then all fields will have C<< _n >> appended to their name, where
+C<n> is the L</repeatable_count> value.
 
 =head2 repeatable_count
 
@@ -301,6 +343,7 @@ Any attributes set will be passed to every repeated Block of elements.
     ---
     elements:
       - type: Repeatable
+        name: my_rep
         attributes: 
           class: rep
         elements:
@@ -309,10 +352,10 @@ Any attributes set will be passed to every repeated Block of elements.
 Calling C<< $element->repeat(2) >> would result in the following markup:
 
     <div class="rep">
-        <input name="foo" type="text" />
+        <input name="my_rep.foo_1" type="text" />
     </div>
     <div class="rep">
-        <input name="foo" type="text" />
+        <input name="my_rep.foo_2" type="text" />
     </div>
 
 See L<HTML::FormFu/attributes> for details.
@@ -324,6 +367,7 @@ The L</tag> value will be passed to every repeated Block of elements.
     ---
     elements:
       - type: Repeatable
+        name: my_rep
         tag: span
         elements:
           - name: foo
@@ -331,10 +375,10 @@ The L</tag> value will be passed to every repeated Block of elements.
 Calling C<< $element->repeat(2) >> would result in the following markup:
 
     <span>
-        <input name="foo" type="text" />
+        <input name="my_rep.foo_1" type="text" />
     </span>
     <span>
-        <input name="foo" type="text" />
+        <input name="my_rep.foo_2" type="text" />
     </span>
 
 See L<HTML::FormFu::Element::block/tag> for details.
@@ -349,6 +393,7 @@ See L<HTML::FormFu::Element::block/auto_id> for further details.
     ---
     elements:
       - type: Repeatable
+        name: my_rep
         auto_id: "%n_%r"
         elements:
           - name: foo
@@ -356,10 +401,10 @@ See L<HTML::FormFu::Element::block/auto_id> for further details.
 Calling C<< $element->repeat(2) >> would result in the following markup:
 
     <div>
-        <input name="foo" id="foo_1" type="text" />
+        <input name="my_rep.foo_1" id="foo_1" type="text" />
     </div>
     <div>
-        <input name="foo" id="foo_2" type="text" />
+        <input name="my_rep.foo_2" id="foo_2" type="text" />
     </div>
 
 =head2 content
@@ -373,7 +418,7 @@ Is a sub-class of, and inherits methods from
 L<HTML::FormFu::Element::Block>, 
 L<HTML::FormFu::Element>
 
-L<HTML::FormFu::FormFu>
+L<HTML::FormFu>
 
 =head1 AUTHOR
 
