@@ -3,10 +3,10 @@ package HTML::FormFu::Plugin;
 use strict;
 use Class::C3;
 
-use HTML::FormFu::Attribute qw/ mk_accessors /;
+use HTML::FormFu::Attribute qw( mk_item_accessors mk_accessors );
 use HTML::FormFu::ObjectUtil qw( populate form parent );
-use Scalar::Util qw/ refaddr /;
-use Carp qw/ croak /;
+use Scalar::Util qw( refaddr );
+use Carp qw( croak );
 
 use overload
     'eq' => sub { refaddr $_[0] eq refaddr $_[1] },
@@ -14,7 +14,7 @@ use overload
     bool => sub {1},
     fallback => 1;
 
-__PACKAGE__->mk_accessors(qw/ type /);
+__PACKAGE__->mk_item_accessors(qw( type ));
 
 sub new {
     my $class = shift;
@@ -25,7 +25,7 @@ sub new {
 
     my $self = bless {}, $class;
 
-    for (qw/ type /) {
+    for (qw( type )) {
         croak "$_ attribute required" if !exists $attrs{$_};
     }
 
@@ -38,6 +38,8 @@ sub new {
     return $self;
 }
 
+sub pre_process { }
+
 sub process { }
 
 sub post_process { }
